/**
 * Solar BackPack v2
 * 
 * Components
 * - DFRobot Beetle 5V Arduino Board
 * - SK6812RGBW based Adafruit NeoPixel Digital RGBW LED Strip 
 * - Loomia Lit Button
 * 
 * Arduino Board Setting: Arduino AVR Boards > Arduino Leonardo
 * 
 * Connections:
 * Arduino :
 * D9 (PWM) connects to a NeoPixel light strip
 * D10 (PWM) connects to the Lumia Button for LED control
 * D11 connects to the Lumia Button for button reading
 * A0 connects to the C (Charging) open drain on the Solar Lipoly Charger
 * A1 connects to the D (Done) open drain on the Solar Lipoly Charger
 * A2 connects to LBO on the PowerBoot Basic
 * 
 * Loomia Button (left to right with the botton on top):
 * Left (LED): GND
 * Center Left (Button): GND
 * Center Right (Button): Arduino Digital Pin D11
 * Right (LED): Arduino Digital PWM Pin D10 
 * 
 */
#include <Adafruit_NeoPixel.h>

// NeoPixels
const int kNeoPixelPin = 9;     // digital PWM pin for neopixels
const int kNumPixels = 15;
const int kBrightness = 32;     // 0 (off) to 255 (max brightness)

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(kNumPixels, kNeoPixelPin, NEO_RGBW + NEO_KHZ800);

// LED modes
const uint8_t kPixelsOff = 0;
const uint8_t kStatusMode = 1;
const uint8_t kTwinkleMode = 2;
const uint8_t kRainbowMode = 3;
uint8_t currentMode = kStatusMode;

uint32_t displayColor = 0;
int currentPixel = 0;           // current strip position
byte cycleCounter = 0;          

// Loomia Button
const int kButtonLedPin =  10;  // digital PWM pin for the LED
const int kButtonPin = 11;      // digital input pin for the button
boolean isButtonPressHandled = false;
int ledBtnState = LOW;          // the current state of the output pin
int fadeAmount = 5;             // how many points to fade the LED by

void setup() {
  pinMode(LED_BUILTIN, OUTPUT); // magic blue LED
  
  // set up NeoPixels
  displayColor = pixels.Color(32, 255, 255);
  pixels.begin();
  pixels.setBrightness(kBrightness);
  pixels.fill(statusColor());
  pixels.show();

  // set up Loomia button
  pinMode(kButtonPin, INPUT_PULLUP);
  pinMode(kButtonLedPin, OUTPUT);
  fadeLedBtnUp();
}

void loop() {
  // handle button presses
  if (isButtonPressHandled == false) {
    isButtonPressHandled = true;
    if (digitalRead(kButtonPin) == 0) {
      // button pressed
      currentMode ++;
      if (currentMode > kRainbowMode) {
        currentMode = kPixelsOff;
      }
      if (currentMode == kPixelsOff) {
        digitalWrite(LED_BUILTIN, LOW);
        pixels.clear();
        pixels.show();
        fadeLedBtnDown();
      } else if (currentMode == kStatusMode) {
        digitalWrite(LED_BUILTIN, HIGH);
        fadeLedBtnUp();
      } else if (currentMode == kTwinkleMode) {
        pixels.clear();
        pixels.show();
      }
    }
  }
  if (digitalRead(kButtonPin)) {
    // button released
    isButtonPressHandled = false;
  }
  // Handle LED Animations
  switch(currentMode) {
    case kStatusMode:
      animateStatus();
      break;
    case kTwinkleMode:
      flashRandom(20);
      break;
    case kRainbowMode:
      // run from 0 to 255
      cycleCounter = (cycleCounter + 1) % 256;
      rainbowCycle(cycleCounter);
      break;
    default:
      break;
  }
  delay(150);
}

void fadeLedBtnUp() {
  int brightness = 0;
  //Serial.println("Fading Up!!");
  while (brightness < 255) {
    brightness = brightness + fadeAmount;
    analogWrite(kButtonLedPin, brightness);
    //Serial.println(brightness);
    // wait for 30 milliseconds to see the dimming effect
    delay(30);
  }
  //Serial.println("Fading Up Done!!!");
}

void fadeLedBtnDown() {
  int brightness = 255;
  //Serial.println("Fading Down!!");
  while (brightness > 0) {
    brightness = brightness - fadeAmount;
    analogWrite(kButtonLedPin, brightness);
    // wait for 30 milliseconds to see the dimming effect
    delay(30);
  }
}

uint32_t statusColor() {
  uint32_t statusColor = pixels.Color(32, 255, 255);
  return statusColor;  
}

// Basic Animation
void animateStatus() {
  displayColor = statusColor();
  if (currentPixel < kNumPixels) {
    currentPixel++;
  } else {
    currentPixel = 0;
  }
  pixels.setPixelColor(currentPixel, displayColor);
  fadePixel(currentPixel-1);
  fadePixel(currentPixel-2);
  fadePixel(currentPixel-3);
  fadePixel(currentPixel-4);
  pixels.show();
}

// Fade a pixel by 50%
void fadePixel(int pixel) {
  uint32_t color = pixels.getPixelColor(pixel);
  uint8_t red = color >> 16;
  uint8_t green = color >> 8;
  uint8_t blue = color;

  int r = red / 2;
  int g = green / 2;
  int b = blue / 2;
  pixels.setPixelColor(pixel, pixels.Color(r, g, b));
}

void flashRandom(int wait) {
  displayColor = statusColor();
  uint8_t red = displayColor >> 16;
  uint8_t green = displayColor >> 8;
  uint8_t blue = displayColor;

  // get a random pixel from the list
  int j = random(pixels.numPixels());
  // now we will 'fade' it in 5 steps
  for (int x=0; x < 5; x++) {
    int r = red * (x+1); r /= 5;
    int g = green * (x+1); g /= 5;
    int b = blue * (x+1); b /= 5;
    
    pixels.setPixelColor(j, pixels.Color(r, g, b));
    pixels.show();
    delay(wait);
  }
  // & fade out in 5 steps
  for (int x=5; x >= 0; x--) {
    int r = red * x; r /= 5;
    int g = green * x; g /= 5;
    int b = blue * x; b /= 5;
    
    pixels.setPixelColor(j, pixels.Color(r, g, b));
    pixels.show();
    delay(wait);
  }
  // pixel will be off when done (it will faded to 0)
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t wheel(byte wheelPos) {
  if(wheelPos < 85) {
   return pixels.Color(wheelPos * 3, 255 - wheelPos * 3, 0);
  } else if(wheelPos < 170) {
   wheelPos -= 85;
   return pixels.Color(255 - wheelPos * 3, 0, wheelPos * 3);
  } else {
   wheelPos -= 170;
   return pixels.Color(0, wheelPos * 3, 255 - wheelPos * 3);
  }
}

void rainbowCycle(byte WheelPos) {
  int idx;
  for(uint16_t i=0; i< pixels.numPixels(); i++) {
    idx = (i * 256 / pixels.numPixels()) + WheelPos;
    pixels.setPixelColor(i, wheel( idx & 255));
  }
  pixels.show();
}
